#!/bin/env python
# This file is: MarilynCasinoPack.py
# This script is a game played in text,
# it contains a "Three shell, two shell, me shell, you shell" game
# and a 20 roll dice game vs. two numbers.  (Men's rule, or Marilyn's)
#
# What the heck caused me to write this?
# http://www.physicsforums.com/showthread.php?t=520738
# On this thread Marilyn get ridiculed, and I have received the same.
# The program simply demonstrates a-priori vs. a-postiori probabilities.
#
# Written by Andrew F. Robinson,(andrewr on physicsforums.com), 1990,2010,2012
# In honor of Marilyn Von Savant's contribution to my educational years.
# Version 1.0
#
# This program *except* my name and honor is dedicated to the
# public domain. You don't have the right to *damage* the program and
# leave my name on it if you go viral with it.
#
# Need Python?  Install instructions:
# Search the original and intricate omni platform provider: www.python.org 
# or for common Windows(tm) OS , I wrote a quick-start here.
# http://www.catholic.com/showpost.php?p=8943837&postcount=20
#
# Oh, and yes, I know I didn't code to standard non-tabbing Python guidlines
# The variables are more Javascript like; but I do try to be consistent.

# ========================= Program editable variables section ==================

# Warning, the following is for obsessive auditors only.... :)
# Edit this next line to True to obtain information on random sequences used
# after a 10x game session.  These can be used in testing and debugging
# program operation.
fullDisclosure=False

# Attempt to restrict average (mean) results to only decisive 
tryIndeterminate=False # Change to True to try non-50/50 indeterminates

# ========================== Begin generic program definitions ================
def RandomPool( possibilities ):
	import random
	"""
	Generate a *reasonably* fair pool of randomized test devices.
	This isolates random generation to *ONE* place in the program and works
	by shuffling a guaranteed balanced array of outcomes for "fair" play.
	It stores the data in arrays, a priori the games, so others may write
	randomness tests, save the values to disk, etc, 
	if they deem it necessary to cross examine/debug the program execution 
	this permits easy access to a random wrapper...	
	(See also: fullDisclosure switch above)	
	"""
	pool=[]
	for i in xrange( 401 * len(possibilities) ):
		pool.extend( possibilities )
		if not i&31: random.shuffle(pool)
	random.shuffle(pool)
	return pool

def DiceToShell( dice ): return (["SHELL",1,2,3,1,2,3])[dice] # map 1-6 to 1-3

def ReverseCoin( face ): return "Tails" if face=="Heads" else "Heads"

def GetChoice(menu):
	"""
	A wrapper for raw input of a menu number.
	Returns a counting number. 
	On Easter I'll .__doc__  you if you know not the Eggs in Python.
	"""
	while True:
		doc=False
		choice = raw_input( "Enter a vaild menu number:" )
		if len(choice)>8 and choice[-8:]==".__doc__":
			doc,choice=(True , choice[:-8])
		try: choice=int(choice)
		except ValueError: choice=0 # invalid number

		if choice>0:
			try:	menu.index(choice)
			except	AttributeError:
				if choice<=menu: return (choice,doc)
			except	ValueError: pass # selection invalid
			else: return (choice,doc)
		print "Invalid choice, try again."

# ==================== Begin Ender's game routines ============================

def ShellGame(_):
	"""
	Play a "Three then eject one Spam can shell game!"
	Load one of three cans with spam on a table, then let user pick one.
	Then, Pythonically eject a different but empty shell off the table
	when the customer finishes choosing. (please miss the customer!)
	Apologize profusely, and do allow (subject to Marilyn shell game 
	rules), the upset customer to re-choose among the two remaining shells.

	Marilyn maintains that the probability of getting a prize from the
	two remaing shells is not 50%/50% *depending* on your *aposteriori*
	choosing choice method.  That is the point of the test.

	This demonstrates how I learned about aposteriori probability changes 
	over 18 years ago, when I lost a serious bet to a mate of mine.
	Learn from my mistakes (!) as the voice of casino experience says *ouch*

	:)
	"""
	onTable={ 1:"Nothing", 2:"Nothing", 3:"Nothing" } # Set shells on table

	global usedDice, usedCoins # Casinos Keep track of used dice and coins.
	whichShell, usedDice = DiceToShell( dicePool[usedDice] ), usedDice+1
	onTable[whichShell]="SPAM AND SpAm sPaM SpAm" # Fill one shell randomly 

	def GetOnTable(): # Pick a shell subroutine
		menu=[]
		print "Shell game Table menu: appetizer is in ONE of:"
		for i,j in onTable.items():
			print "shell and tin can *",i,"*, ",
			menu.append(i)
		print "\n"
		while True:
			choice,_=GetChoice(menu)
			if _: 
				print ShellGame.__doc__
				continue
			break
		return choice

	choice=GetOnTable() # first, Let the customer calmly choose a shell

	# Flip a coin in preparation for violent ejaculation
	headsOrTails,usedCoins = coinPool[usedCoins], usedCoins+1

	# Do a quick postiori analysis for a dangerous random eject
	if onTable[choice]=="Nothing": headsOrTails="Heads"# remove possibility 

	for i,j in onTable.items():
		if i==choice: continue # Don't ever remove the user's choice!
		if j=="Nothing":  # This is empty, and thus not illegal to bomb 
			if headsOrTails == "Heads": # Randomly set off a shell 
				del onTable[i] # Shell is NOW GONE off table.
				print "\nI'm SO Sorry!"
				print "Can.an.d shell #",i," Just BLEW off!!!"
				print "Thankfully it held no prize!"
				print "Whew, now, you may re-choose for prize"
				print
				break
			else:
				headsOrTails=ReverseCoin( headsOrTails )

	# There are now TWO shells left, and the prize is still available 
	# Notice there was NO SWITCHEROO.  Just a safe pointless ejaculation.
	# Now the customer still doesn't KNOW where the item is, they guess ??
	
	choice=GetOnTable()
	print "You found ",onTable[choice]," for a prize"
	return ( onTable[choice] != "Nothing" )	# Win Spam=True, Nothing=False
# END of shell game.


# Begin dice game
# These are the numbers attested to in the OP from Parade (tm) magazine.
ones  =[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1] # The actual parade magazine
parade=[6,6,2,3,4,4,4,1,5,3,6,1,2,5,5,6,3,1,5,2] # Question's sequences

countOnes=0   # counters for statistical hits among sequences
countParade=0
countMarilyn=0 # and a separate counter for Marilyns interpretation

def DiceGame(params):
	"""
	This is an interpretation of Marilyn's 20 dice question as posed
	on this thread:

	http://www.physicsforums.com/showthread.php?t=520738

	Let Q represent the person who asked Marilyn the question. (Quelle)
	There are only three solid pieces of evidence usable as witnesses
	to judge Marilyn's response esp. as Lauren has not added additional
	information to the OP...

	1) Someone is to roll a dice 20x according to an undisclosed plan.
	2) Two results consisting of a sequence of 20x digits are presented.
	3) The question is merely which of the two sequences is more likely.

	The question does not specify what the plan is, nor the
	who/what/how/when or Why these two particular sequences were chosen as
	representative.  Hence, the interpretation is open to the imagination.

	Formally, the question states: "Say *you* plan", which has a colloqual
	meaning and is NOT fixed to one idea.

	http://forums.catholic.com/showpost.php?p=8272384&postcount=1342

	"I can tell you what I didn't mean ... What I meant was you (not you
	personally) should be able to answer a straight answer to a straight
	question." ...

	In reality: the test sequences of Q may have been generated many ways;
	
	1a) Q rolled '11111'20x, and made up the '6623...' sequence.
	2a) Q rolled '6623...' and made up the '1111'20x sequence.
	3a) Q rolled both sequences and reported them. 
	4a) Q invented/discovered both unmbers from non-dice rolls.

	Observing the possibilities, 1-3, and the infinitive verb of the
	question "to roll" which has no TENSE; One must allow that Q
	could potentially be asking a variation of the following.
	
	1b) Q wants to know the probability of her rolls/sqq. being *repeated*
	2b) Q wants to know if Marilyn can detect an imaginary sequence vs.
	    an actual rolling of a dice which Q did to determine a sequence. 
	    (eg: please note the "plan" was *not* disclosed in the OP.)

	The following program replicates the *NATURE* of the question,
	and allows the gambler to take MARILYN'S position and gamble
	on the outcome as a statistical test of Marilyn's analysis, or of
	micromass's statements (4a-1b) still not knowing the plan, etc... 
	"""

	global usedDice, usedCoins

	def Indeterminate():
		""" if Invalid :. Q still has 'even' odds of being """
		print "Indeterminate result :. bias->unbiased"
		if tryIndeterminate: return None
		# Substitute an unbiased 50% as equivalent to "Indeterminate"
		global usedCoins
		usedCoins+=1
		return (coinPool[usedCoins-1]=="Heads")

	def Result( win ):
		""" Return and print whether a winner or not.  """
		print ":. bias -->",("accepting" if win else "rejecting")," bet"
		return win

	auto,directGamble,forceAPriori=params

	# Start with the a-priori values given by Q, but unordered eg: 4a-1b)
	choices=[ones,parade] if coinPool[usedCoins]=="Heads" else [parade,ones]
	usedCoins+=1

	print "Say you plan to roll a dice 20x."

	# Secretly determine a *randomly* chosen undisclosed PLAN, as Q did not
	# specifically say what the PLAN was, nor its relation to the numbers.
	# as the default plan does seem to be more popular, grant the detractors
	# a higher probability by contstruction, and a lower probability to the
	# other possible connotations of the words used by Q in the question.

	# Notice, actual dice rolls *can* even replicate the original Q EXACTLY
	# Therefore, the meaning of the question is definitely included in all
	# plans until Q explains ANY INFORMATION s/he gave to Marilyn not
	# included on the physics forums.  eg: We do not know, until Q tells us
	# Whether her numbers might change if she asked the question *again* or
	# not.  We simply don't know what the numbers are based on.

	# Choose one OR two possible replacement numbers.
	# Decide a-priori or postiori whether and how the second roll is to
	# be used.  This, I believe, will likely affect probability of outcomes
	# This type of subtlety is also why I think Marilyn *MIGHT* have given
	# the second answer; and micromass and others appear to deny she could
	# be answering.

	# First alternate is chosen a-priori, ONLY.
	_alt0,usedDice=dicePool[usedDice:usedDice+20],usedDice+20

	# Second dice may be a-posterori or force chosen a-priori.
	# Note: Forcing it a-priori happens to be a statistically *MALE*  
	# interpretation of Marilyn's meaning. I don't know if it is appropriate
	# to allow this or not, as I don't have data from women on what she said
	# But if I don't allow it, I have good reason to believe the code will
	# be suppressed by PF moderators who show a predictable pattern of
	# ultimatum giving and who are presently strongly suggesting they
	# would suppress this code otherwise.  My apologies if I have made an
	# error by allowing this; I am not sure what else to do at this time.
	# Please note my reservations until I am able to resolve the issue.

	_alt1=dicePool[usedDice:usedDice+20] # A potentially unused dice roll
	_aPriori,usedCoins= (coinPool[usedCoins]=="Heads"),usedCoins+1
	if forceAPriori: _aPriori=True # Override random choice
	if _aPriori: usedDice+=20 # The dice roll will, in fact, be a-priori

	# Synthesize a random sequence, as an example of an a-posteriori
	_alt2=_alt1[:-10]
	_alt2.extend(_alt0[:10])

	# NOTE: These arbitrary numbers DO NOT have to be random or uncorrelated
	# Their actual source, whether parade magazine, random generator,
	# me myself, and I -- are *all* irrelevant to the question,
	# in the parade question, their source and priori/posteriori nature
	# are not defined.  I simply don't know where, how or when they were
	# found by Q due to the English grammar of the question.
	# In my implementaion, I am just doing random rolls for simplicity.
	# There are, of course, other choices.

	# Sometimes, the same number can be used twice for both choices...
	# I make no apology for any correlations between *ANY* dice outcomes
	# and their a priori or posteriori nature.  This is all strictly legal
	
	# I am interested in the ethics, though, of those who judge Marilyn
	# and what formal, rational reasonning they may come up with to
	# justify a restrictive English interpretation of the potentially 
	# leading question asked in Parade(tm) magazine of Marilyn.
	# Scientific ethics are something needing study for the sake of
	# reaching consensus between world religions, scientific establishments
	# and healing the wounds of historical abuses and present day revenge.
	# Judge for yourself what is right and wrong

	# Switch supposedly equal probability sequences and see what happens
	# This may not make a difference depending on how the user chooses
	# The outcome
	if not auto == 3:  # If running a strict numbers test, don't change
	    if coinPool[usedCoins]=="Heads" and coinPool[usedCoins+1]=="Tails":
	        choices[1],aPriori=(_alt0,False) if _aPriori else (_alt1,True)
	        usedCoins+=2

	    if coinPool[usedCoins]=="Heads" and coinPool[usedCoins+1]=="Tails":
		choices[0],aPriori=(_alt2,True ) if _aPriori else (_alt0,True)
	        usedCoins+=2

	# Now I'll hide evidence of the arbitrary plan, because the outcome's
	# relationship to this plan is not specified in the leading question
	# to Marilyn, either.

	del _aPriori, _alt0, _alt1, _alt2

	# Setup/sting game is complete, now begin the actual sampling
	print "Stay calm, don't TENSE up!"
	print "Which of these is more likely *to be* rolled? (whenever)"

	menu=[]
	print "1)",choices[0]
	print "2)",choices[1]
	menu.extend([1,2])
	if not auto and not directGamble:
		print "3) neither but restrict gamble to Q's original numbers"
		print "4) neither AND bet Marilyn vs. BOTH Q's specific numbers"
		menu.extend([3,4])

	while True:
		choice,_=(GetChoice(menu)) if not auto else (auto,"")
		if not _: break
		print DiceGame.__doc__ 

	if choice<3: choice-=1 # convert menu 1 & 2 to choices[] index

	# Now, test the user's choice against a pre-PLANNED dice roll
	decidingRoll,usedDice=dicePool[usedDice:usedDice+20],usedDice+20
	print ":)",decidingRoll, "(APost... roll)"

	# Test for match between roll and arbitrary planned numbers
	matches= [ (decidingRoll==choices[0]) , (decidingRoll==choices[1]) ]
	match=   matches[0] or matches[1]

	# -------- Calculate & report the outcome of the experiment ----------

	# Find out as a sample statistic, eg: how often they came up the same.
	# It is only "successes" which are measured, not nullness.

	global countOnes, countParade, countMarilyn # Do cumulative accounting

	# Account for special recognizable (signatureized) sequences *FIRST*
	# but only allow individual's "psychological" gambling after this 

	checkQs=False # Assume at first Q's choices are not used, but check!
	# They could be either randomly rolled, or arbitrarily chosen. 
	try:    checkQs=bool(choices.index(ones)+1)   # Testing Q's 1111...?
	except ValueError: pass # No 
	try:    checkQs=bool(choices.index(parade)+1) # Testing Q's 6623...?
	except ValueError: pass # No 

	# Tally statistics on dice roll occurrances of any of parade's Q or M's
	# regardless of applicability to the variation desired for sampling.
	if checkQs:
		# if the dice rolled either of Q's #'s record it as a statistic
		if decidingRoll==ones: countOnes+=1 # ones actually found ?!?!
		else:
			countMarilyn+=1 # Marilyn's random non 1's bet "rolled"
			if decidingRoll==parade: countParade+=1 # Q's 6623...?!  

	# Use statistics in accordance with auditor's desire to test proposition
	# An indeterminant result is to return an unbiased coin flip for now

	if choice==3 or choice==4: # Test for "neither" in one of two ways.
		# HOWEVER, exclude false positives or negatives.
		if (	not checkQs # Avoid temporal sampling of bias and
			or  (   choice==3 and # for strict Q's equality test. 
				not (countOnes+countParade) # Don't accept NULL
			)
		): return Indeterminate()

	if choice==3:  # Choose neither, but restricted to Q's specific sqq. 
		if countOnes != countParade:
			print "1's=%d vs 6623..'s=%d"%(countOnes,countParade)
		return Result(countOnes==countParade)

	# Chose neither,  and put Marilyns response to the test, too
	# Be extra conservative -- pit BOTH Q's sqq. against Marilyn.
	
	if choice==4:
	    if not match: return Indeterminate()
	    return Result( countOnes+countParade < countMarilyn )

	# With special sequences out of the way, individual human Gambles !!!
	# Human driven gambling becomes available, eg: less volatile users
	# can do General betting against TWO Aribtrary numbers to see
	# If they can determine (by telepathy, reason, etc.) which is more likely

	if not (matches[choice] ^ matches[choice-1]): return Indeterminate()
	return Result( matches[choice] )	

# ================================= A toy timer =================================

def RealTimeClock( stopAfter ):
	"""
	A relaxing puzzle that is of general estimation & counting, interest.
	This clock progesses in value every ~1/7 second 
	(eg: in stochastic, but imaginary Al_ph_1 decayings which are *slow*;))
	if the clock will not terminate iteslf, I give a courtesy notice.

	For checking an estimation of the value of a siderial year, there is
	a program on CAF (forums) that will translate a NaugtTrek real time into 
	its equivalen count, which can be divided by 7 to see if it is near a
	year worth of seconds or not.

	But, I'll let you scavenger hunt for it...
	
	The clock is only memory limited in theory; so there is no Y2K bug,
	or even a chance of it theoretically missing ra eal future years.

	If I made a mistake, and there IS a real future date ( decimal number )
	that my clock would miss even on a perfect Touring Machine, Check the
	bottom of the thread this program is listed in to see if I have admitted
	it, and if not -- perhaps contacting me is in order. ;)
	0 is not an error ... nor any trailing ...99999999999999999999[9bar]s
	:).  This is just a decimal point containing clock -- not GOD almighty!
	"""

	import time
	tick = 1./7 # Subsecond time intervel for program looping
	from sys import stdout # Get flush function for clock, only.
	if not stopAfter:
		print "A Relaxing frivolity, meditation and Estimation clock"
		print "As a Light hearted puzzle, can you guess-estimate a"
		print "value this crazy clock will read in 1 siderial"
		print "Earth year?  It is decimally predictable in all digits."
		print	
		print "of course you could locate a year by a LONG meditation,"
		print "(or use Ctrl-C to quit early. :) )"
	else:
		print "JUST WAIT",
		print stopAfter," SECOND"+("!"if stopAfter==1 else"S.")
		
	places,cap={ 0:".", -1:0, 1:0 },1
	dHour,seconds=359.7,0 # Decimal hour=1/10th an hour (3600seconds/10)
	
	while True:
		index=1
		time.sleep( tick )
		seconds += tick
		while True:
			places[index]=(places[index]+1)%10
			if places[index]: break
			index=-index if index>0 else -index+1
			if index>cap:cap,places[index],places[-index]=index,0,0
		print "NaugtTrek stareDate is: ",
		# Next convert dictionary to a printable concatinated string.
		s="".join(map(str,[places[cap-i] for i in xrange(0,cap*2+1)]))
		print s, # print time

		if stopAfter and seconds >= stopAfter-0.01:
			print  "\n"
			return "\n\n\n\n\n\n\n\n\n\n"
		if seconds > dHour:
			print ("Has it really been over %4.1f Hours!?"%
					(seconds/3597.) # Stochastic delay adj
			),
			dHour+=359.7
		print "\r",
		stdout.flush()

#============================== Setup the CASINO !!!!!!!!!!! ==================

print "Generating a lot of potential dice rolls."
dicePool=RandomPool( [1,2,3,4,5,6] )
usedDice=0
print "Generating a lot of potential coin flips."
coinPool=RandomPool(  ["Heads","Tails"] )
usedCoins=0

RealTimeClock(1)
print "Ender's wiggin Marilyn Casino is now open."

dice="Plan to roll 20 dice and see if it comes up as\n     " 
games=[ None, #0
	(ShellGame,None), #1
	(DiceGame,[3,False,False]), # auto, directGamble, forceAPriori 
	(DiceGame,[4,False,False]),
	(DiceGame,[False,False,True]),
	(DiceGame,[False,True,False]),
	(RealTimeClock,None),
	(quit,None), #7
	(quit,None)  #8
      ]

#======================== Keep a statistic of wins and losses =================

wins=0
attempts=0

# Begin playing !

while True:
	print
	print "What would you like to do? 1-5 are minimum of 10x games each"
	print "1) shell game: demo of meaning of aPosteriori modification"
	print "2) play 20x dice, auto choose to test *equal* probability of Q"
	print "3) play 20x dice, auto choose to bet like Marilyn" 
	print "4) play 20x dice, both basic tests, but aPriori only arbitraries"
	print "5) play 20x dice, see if you can item-wise beat the casino"
	print " ------ Or NOW for something totally different ------"
	print "6) Relax by Meditating on a 'Real' time decay clock"
	print "7) Just Leave Me Alone and Quit ALREADY!"
	print "-----------------------------------------------------"
	print "8,8+) These are dummy tests for al_ph a naught-y dummys(Not you)"

	choice,_=GetChoice(77)
	if _:
		print games[choice][0].__doc__
		RealTimeClock(30)
		continue
	print "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"

	if choice >= 1 and choice <= 7:
		localWins=0
		exclude=0
		for i in xrange(1,11):
			result=(games[choice])[0]((games[choice])[1])
			try: localWins += int(result) # Determinate True=1,F=0
			except TypeError: exclude+=1 # Invalid results exclude

			print "\nScore %3.0f%% wins in this set"%(
				localWins/(i+0.)*100
			),"--------",i,"of 10","\n"
			RealTimeClock(5)
	
		wins+=localWins
		attempts+=i-exclude
		pct = wins/(attempts+0.) if attempts else 0.
		print "_/-_/-_/-_/-_/-_/-_/-_/-_/-_/-_/-_/-_/-_/-_/-_/-_/-_/-_"
		print "Your average wins are: %3.0f%%\n"%(pct*100.)
		if (pct<0.35 and choice==1):
			print "Savant women cans *ALWAYS* change their mind."
			print "Do you DARE try act like THAT kind of woman?"
			print "Careful: Its a PACT for 66.6 devilish percent!"
			print "(Truly, Gambling is an addiction, esp in Lent )"

		print RealTimeClock(5)
	
		if fullDisclosure:
			print "FULL disclosure of random numbers"
			print "The just used dice rolls were:"
			print dicePool[:usedDice]
			print "The just used Coin flips were:"
			print coinPool[:usedCoins]
			print RealTimeClock(7)
	else:
		print "Have you ever seen the M.Python apple wreck predictor?"
		print "trust me --> out the window it just jumped..."
		print "Try a lower number...."
		print "eg: 7  means disowning Cain who is marked unAble 7x."
		print "and 13 means 12 + a 1 very incompitent traitor."
		print "So, If in doubt just type 7 and press enter."
		print RealTimeClock(choice)

	del coinPool[:usedCoins] # Remove used coins from the pool
	del dicePool[:usedDice]  # Remove used dice from the pool
	usedCoins,usedDice=0,0 # Inform of freshened pool, no history

	if len( coinPool )<20 or len( dicePool )<200:
		print "The Casino is arbitrarily closed, NOW Quit!"
		print "This man-a-andrew wants you to spam-scrham"
		quit()

